#ifndef GC_WRAPPER_H
#define GC_WRAPPER_H

#include "gc/pointer.h"
#include "gc/object.h"
#include "gc/gc.h"

namespace gc {

template<typename T>
class Wrapper {
protected:
	class WrappedObject : public Object {
	protected:
		T *mobj;
	public:
		WrappedObject(T *obj_) : mobj(obj_) {}
		virtual ~WrappedObject() { delete mobj; }
		T* obj() { return mobj; }
		const T* obj() const { return mobj; }
	};

	Wrapper() : wr(sys->createPtr<WrappedObject>()) {}		//defaults to null
public:
	GlobalPointer<WrappedObject> wr;
	Wrapper(const Wrapper &o) : wr(o.wr) {}
	//pass the wrapped object back -- and give the wrapper ownership of the ptr (to-be-deleted when the wrapper is deleted... to be deleted upon un-ref-ing)
	Wrapper(T *ptr) : wr(sys->createPtr<WrappedObject>(new WrappedObject(ptr))) {}
	
	T *operator->() { return wr->obj(); }
	const T *operator->() const { return wr->obj(); }
	operator T*() { return wr->obj(); }
	operator const T*() const { return wr->obj(); }
};

};

#endif	//GC_WRAPPER_H
